@@ -4,6 +4,6 @@ dir=$(dirname "$0")
 cd "$dir/.."
 
 find . -name __pycache__ -type d | while read d
-  do rm -rf "$d"
+  do rm -rfv "$d"
 done
-rm -rf .pytest_cache build dist src/*.egg-info
+rm -rfv .pytest_cache build dist src/*.egg-info
