@@ -11,7 +11,7 @@ Use Java classes from Python:
 Use Maven artifacts from remote repositories:
 
     >>> from scyjava import config, jimport
-    >>> config.add_option('-Djava.awt.headless=true')
+    >>> config.enable_headless_mode()
     >>> config.add_repositories({
     ... 'scijava.public': 'https://maven.scijava.org/content/groups/public',
     ... })
@@ -66,16 +66,13 @@ Convert Python collections to Java:
     >>> jset.toString()
     '[1, 2, 3]'
 """
+
 import logging
 from functools import lru_cache
 from typing import Any, Callable, Dict
 
-from scyjava._arrays import (  # noqa: F401
-    is_arraylike,
-    is_memoryarraylike,
-    is_xarraylike,
-)
-from scyjava._convert import (  # noqa: F401
+from ._arrays import is_arraylike, is_memoryarraylike, is_xarraylike
+from ._convert import (
     Converter,
     JavaCollection,
     JavaIterable,
@@ -94,30 +91,34 @@ from scyjava._convert import (  # noqa: F401
     to_java,
     to_python,
 )
-from scyjava._java import (  # noqa: F401
-    JavaClasses,
+from ._jvm import (  # noqa: F401
+    available_processors,
+    gc,
     is_awt_initialized,
-    is_jarray,
     is_jvm_headless,
-    isjava,
-    jarray,
-    jclass,
     jimport,
-    jinstance,
-    jstacktrace,
     jvm_started,
     jvm_version,
+    memory_max,
+    memory_total,
+    memory_used,
     shutdown_jvm,
     start_jvm,
     when_jvm_starts,
     when_jvm_stops,
 )
-from scyjava._script import enable_python_scripting  # noqa: F401
-from scyjava._versions import (  # noqa: F401
-    compare_version,
-    get_version,
-    is_version_at_least,
+from ._script import enable_python_scripting
+from ._types import (
+    JavaClasses,
+    is_jarray,
+    isjava,
+    jarray,
+    jclass,
+    jinstance,
+    jstacktrace,
+    numeric_bounds,
 )
+from ._versions import compare_version, get_version, is_version_at_least
 
 __version__ = get_version("scyjava")
 __all__ = [
@@ -174,10 +175,15 @@ def __getattr__(name):
 
 
 def _initialize_converters():
+    _logger.debug("Initializing type converters")
+
     for converter in _stock_java_converters():
         add_java_converter(converter)
+    _logger.debug("Java converters:{'\n-'.join(java_converters)}")
+
     for converter in _stock_py_converters():
         add_py_converter(converter)
+    _logger.debug("Python converters:{'\n-'.join(py_converters)}")
 
 
 when_jvm_starts(_initialize_converters)
